#!/bin/bash

set -e

echo "starting"
cfgFile=/usr/share/ukui-lite-firstboot/firstboot.cfg
#logFile=/tmp/firstboot/runscripcommand.log
screensaverfile=/usr/share/glib-2.0/schemas/org.ukui.screensaver.gschema.xml
powerfile=/usr/share/glib-2.0/schemas/org.ukui.power-manager.gschema.xml

userName=
password=
autoLogin=
userDefGroup=

get_value()
{
    indexStr=$1
    valueStr=

    valueStr=$(grep "^$indexStr=" "$cfgFile")
    valueStr=${valueStr##*=}
    echo "${valueStr}"
}

get_value_bytearray() {
    local key=$1
    local value
    value=$(grep "^$key" "$cfgFile")

    # TODO, 连续空格有问题
    if ! egrep -q '^password=.?@ByteArray' $cfgFile; then
        echo "${value#*=}"
        return 0
    fi

    value1=$(echo "${value#*\(}")
    echo "${value1%)*}"
}

set_user()
{
    userName=$(get_value "username")
    password=$(get_value_bytearray "password")
    autoLogin=$(get_value "autologin")
    KUID=1000
    #oldusername=`getent passwd |grep 1001|cut -d: -f1`
    #msg "Set user $userName"
    #usermod -l "${userName}" "${oldusername}"
    oldname=$(awk -F: '/1000:1000/{print $1}' /etc/passwd)
    if [ -n "${oldname}" ]; then
	    userdel -r "${oldname}"
	    BEGIN=$(grep -a -n "^oem:" /etc/uid_list | awk -F: '{print $1}')
	    sed -i "$((BEGIN+1)),$ d" /etc/uid_list
    fi

    useradd -m -s /bin/bash -u "${KUID}" "${userName}"
    usermod -c "${userName}" "${userName}"
    echo "$userName:$password" | chpasswd
}

add_groups()
{
    userName=$(get_value "username")
    userDefGroup=(adm cdrom sudo dip plugdev users lpadmin sambashare debian-tor libvirtd lxd)

    for item in ${userDefGroup[*]}; do
        adduser $userName $item || true
    done
}

set_auto_login()
{
    if [[ "${autoLogin}" == "1" ]]; then
        if [[ -d /etc/lightdm ]]; then
            LightDMCustomFile=/etc/lightdm/lightdm.conf
            autoLoginParam="autologin-guest=false\n\
            autologin-user=$userName\n\
            autologin-user-timeout=0"

            if ! grep -qs '^autologin-user' ${LightDMCustomFile}; then
                if ! grep -qs '^\[Seat:\*\]' ${LightDMCustomFile}; then
                    echo '[Seat:*]' >>${LightDMCustomFile}
                fi
                sed -i "s/\[Seat:\*\]/\[SeatDefaults]\n${AutologinParameters}/" ${LightDMCustomFile}
            # oem config scenario
            else
                #sed -i "s/^\(\(str  *\)\?autologin-user\)=.*$/\1=${USERNAME}/g;" /etc/lightdm/lightdm.conf
                sed -i "s/^autologin-user=.*$/autologin-user=${userName}/g" ${LightDMCustomFile}
            fi
        fi
        if [[ -f /lib/systemd/system/accounts-daemon.service ]]; then
                systemctl restart accounts-daemon || true
        fi
    else
        if [[ -f /etc/lightdm/lightdm.conf ]]; then
            rm -f /etc/lightdm/lightdm.conf
        fi
    fi
}

set_hostname()
{
    HOSTNAME=$(get_value "hostname")
    echo "${HOSTNAME}" >/etc/hostname
    hostnamectl set-hostname "${HOSTNAME}"

    echo -e "127.0.0.1\tlocalhost

127.0.1.1\t${HOSTNAME}
# The following lines are desirable for IPv6 capable hosts
::1     ip6-localhost ip6-loopback
fe00::0 ip6-localnet
ff00::0 ip6-mcastprefix
ff02::1 ip6-allnodes
ff02::2 ip6-allrouters
" >/etc/hosts

  # bug 39277, 重启蓝牙服务
  if [[ -f /lib/systemd/system/bluetooth.service ]]; then
    /bin/systemctl restart bluetooth.service || true
  fi
}

delete_oem_user_and_config() 
{
    if getent passwd 'oem' >/dev/null; then
        if [ -f /etc/sudoers.d/oem ]; then
            rm -f /etc/sudoers.d/oem
        fi
        pkill -u 'oem' || true
        userdel --force --remove 'oem' || true
    fi

    if [ -f /usr/share/lightdm/lightdm.conf.d/96-ukui-lite-firstboot.conf ]; then
        rm -f /usr/share/lightdm/lightdm.conf.d/96-ukui-lite-firstboot.conf
    fi

    if [ -f /usr/share/xsessions/ukui-lite-firstboot.desktop ]; then
        rm -f /usr/share/xsessions/ukui-lite-firstboot.desktop
    fi

}

#key=$1,value=$2,file=$3
set_xml_value()
{
    keylinenum=`sed -n "/name=\"$1\"/=" $3`
    valuelinenum=$(($keylinenum + 1))
    #echo $keylinenum $valuelinenum
    if [[ "$1" == "button-power" ]]; then
        sed -r -i "${valuelinenum}s/(<default>).*(<\/default>)/\1"\'$2\'"\2/" $3
    else
        sed -r -i "${valuelinenum}s/(<default>).*(<\/default>)/\1$2\2/" $3
    fi
}

set_screensaver()
{
    idleDelay=$(get_value "idle-delay")
    lockEnabled=$(get_value "lock-enabled")
    closeActivationEnabled=$(get_value "close-activation-enabled")
    sleepActivationEnabled=$(get_value "sleep-activation-enabled")

    if [[ -f "$screensaverfile" ]]; then
        #echo "file is exist"
        set_xml_value "idle-delay" $idleDelay $screensaverfile
        if [[ "${lockEnabled}" == "1" ]]; then
            set_xml_value "lock-enabled" "true" $screensaverfile
        else
            set_xml_value "lock-enabled" "false" $screensaverfile
        fi
        if [[ "${closeActivationEnabled}" == "1" ]]; then
            set_xml_value "close-activation-enabled" "true" $screensaverfile
        else
            set_xml_value "close-activation-enabled" "false" $screensaverfile
        fi
        if [[ "${sleepActivationEnabled}" == "1" ]]; then
            set_xml_value "sleep-activation-enabled" "true" $screensaverfile
        else
            set_xml_value "sleep-activation-enabled" "false" $screensaverfile
        fi
    else
        echo "The file does not exist! ($screensaverfile)"
        exit 1
    fi
}

set_power()
{
    buttonPower=$(get_value "button-power")
    sleepComputerAc=$(get_value "sleep-computer-ac")
    sleepComputerBattery=$(get_value "sleep-computer-battery")
    sleepDisplayAc=$(get_value "sleep-display-ac")
    sleepDisplayBattery=$(get_value "sleep-display-battery")

    if [[ -f "$powerfile" ]]; then
        #echo "file is exist"
        set_xml_value "button-power" $buttonPower $powerfile
        set_xml_value "sleep-computer-ac" $sleepComputerAc $powerfile
        set_xml_value "sleep-computer-battery" $sleepComputerBattery $powerfile
        set_xml_value "sleep-display-ac" $sleepDisplayAc $powerfile
        set_xml_value "sleep-display-battery" $sleepDisplayBattery $powerfile
    else
        echo "The file does not exist! ($powerfile)"
        exit 1
    fi
    
}

set_user 
add_groups 
set_auto_login 
set_hostname
set_screensaver
set_power
#delete_oem_user_and_config
systemctl enable delete-oem-user-for-firstboot.service
glib-compile-schemas /usr/share/glib-2.0/schemas/
